@@ -13,35 +13,44 @@
 # import types so that we can reference ListType in sphinx param declarations.
 # We can't just use list, because sphinx gets confused by
 # openstack.resource.Resource.list and openstack.resource2.Resource.list
+
+import contextlib
+import sys
 import types  # noqa
 import warnings
 
 import jsonpatch
 
-from openstack.cloud import _utils
+from openstack.baremetal.v1._proxy import Proxy
 from openstack.cloud import exc
-from openstack import utils
+
+
+def _normalize_port_list(nics):
+    ports = []
+    for row in nics:
+        if isinstance(row, str):
+            address = row
+            row = {}
+        elif 'mac' in row:
+            address = row.pop('mac')
+        else:
+            try:
+                address = row.pop('address')
+            except KeyError:
+                raise TypeError(
+                    "Either 'address' or 'mac' must be provided "
+                    "for port %s" % row
+                )
+        ports.append(dict(row, address=address))
+    return ports
 
 
 class BaremetalCloudMixin:
-
-    @property
-    def _baremetal_client(self):
-        if 'baremetal' not in self._raw_clients:
-            client = self._get_raw_client('baremetal')
-            # Do this to force version discovery. We need to do that, because
-            # the endpoint-override trick we do for neutron because
-            # ironicclient just appends a /v1 won't work and will break
-            # keystoneauth - because ironic's versioned discovery endpoint
-            # is non-compliant and doesn't return an actual version dict.
-            client = self._get_versioned_client(
-                'baremetal', min_version=1, max_version='1.latest')
-            self._raw_clients['baremetal'] = client
-        return self._raw_clients['baremetal']
+    baremetal: Proxy
 
     def list_nics(self):
         """Return a list of all bare metal ports."""
-        return [nic._to_munch() for nic in self.baremetal.ports(details=True)]
+        return list(self.baremetal.ports(details=True))
 
     def list_nics_for_machine(self, uuid):
         """Returns a list of ports present on the machine node.
@@ -51,13 +60,11 @@ class BaremetalCloudMixin:
         :returns: A list of ports.
         """
         # TODO(dtantsur): support node names here.
-        return [nic._to_munch()
-                for nic in self.baremetal.ports(details=True, node_id=uuid)]
+        return list(self.baremetal.ports(details=True, node_id=uuid))
 
     def get_nic_by_mac(self, mac):
         """Get bare metal NIC by its hardware address (usually MAC)."""
-        results = [nic._to_munch()
-                   for nic in self.baremetal.ports(address=mac, details=True)]
+        results = list(self.baremetal.ports(address=mac, details=True))
         try:
             return results[0]
         except IndexError:
@@ -66,10 +73,9 @@ class BaremetalCloudMixin:
     def list_machines(self):
         """List Machines.
 
-        :returns: list of ``munch.Munch`` representing machines.
+        :returns: list of :class:`~openstack.baremetal.v1.node.Node`.
         """
-        return [self._normalize_machine(node)
-                for node in self.baremetal.nodes()]
+        return list(self.baremetal.nodes())
 
     def get_machine(self, name_or_id):
         """Get Machine by name or uuid
@@ -79,11 +85,11 @@ class BaremetalCloudMixin:
 
         :param name_or_id: A node name or UUID that will be looked up.
 
-        :returns: ``munch.Munch`` representing the node found or None if no
-            nodes are found.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: The node found or None if no nodes are found.
         """
         try:
-            return self._normalize_machine(self.baremetal.get_node(name_or_id))
+            return self.baremetal.find_node(name_or_id, ignore_missing=False)
         except exc.OpenStackCloudResourceNotFound:
             return None
 
@@ -92,8 +98,8 @@ class BaremetalCloudMixin:
 
         :param mac: Port MAC address to query in order to return a node.
 
-        :returns: ``munch.Munch`` representing the node found or None if the
-            node is not found.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: The node found or None if no nodes are found.
         """
         nic = self.get_nic_by_mac(mac)
         if nic is None:
@@ -116,8 +122,8 @@ class BaremetalCloudMixin:
         :param timeout: Integer value, defautling to 3600 seconds, for the
             wait state to reach completion.
 
-        :returns: ``munch.Munch`` representing the current state of the machine
-            upon exit of the method.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: Current state of the node.
         """
 
         return_to_available = False
@@ -131,37 +137,63 @@ class BaremetalCloudMixin:
                 raise exc.OpenStackCloudException(
                     "Refusing to inspect available machine %(node)s "
                     "which is associated with an instance "
-                    "(instance_uuid %(inst)s)" %
-                    {'node': node.id, 'inst': node.instance_id})
+                    "(instance_uuid %(inst)s)"
+                    % {'node': node.id, 'inst': node.instance_id}
+                )
 
             return_to_available = True
             # NOTE(TheJulia): Changing available machine to managedable state
             # and due to state transitions we need to until that transition has
             # completed.
-            node = self.baremetal.set_node_provision_state(node, 'manage',
-                                                           wait=True,
-                                                           timeout=timeout)
+            node = self.baremetal.set_node_provision_state(
+                node, 'manage', wait=True, timeout=timeout
+            )
 
         if node.provision_state not in ('manageable', 'inspect failed'):
             raise exc.OpenStackCloudException(
                 "Machine %(node)s must be in 'manageable', 'inspect failed' "
                 "or 'available' provision state to start inspection, the "
-                "current state is %(state)s" %
-                {'node': node.id, 'state': node.provision_state})
+                "current state is %(state)s"
+                % {'node': node.id, 'state': node.provision_state}
+            )
 
-        node = self.baremetal.set_node_provision_state(node, 'inspect',
-                                                       wait=True,
-                                                       timeout=timeout)
+        node = self.baremetal.set_node_provision_state(
+            node, 'inspect', wait=True, timeout=timeout
+        )
 
         if return_to_available:
-            node = self.baremetal.set_node_provision_state(node, 'provide',
-                                                           wait=True,
-                                                           timeout=timeout)
+            node = self.baremetal.set_node_provision_state(
+                node, 'provide', wait=True, timeout=timeout
+            )
 
-        return self._normalize_machine(node)
+        return node
 
-    def register_machine(self, nics, wait=False, timeout=3600,
-                         lock_timeout=600, **kwargs):
+    @contextlib.contextmanager
+    def _delete_node_on_error(self, node):
+        try:
+            yield
+        except Exception as exc:
+            self.log.debug(
+                "cleaning up node %s because of an error: %s", node.id, exc
+            )
+            tb = sys.exc_info()[2]
+            try:
+                self.baremetal.delete_node(node)
+            except Exception:
+                self.log.debug(
+                    "could not remove node %s", node.id, exc_info=True
+                )
+            raise exc.with_traceback(tb)
+
+    def register_machine(
+        self,
+        nics,
+        wait=False,
+        timeout=3600,
+        lock_timeout=600,
+        provision_state='available',
+        **kwargs
+    ):
         """Register Baremetal with Ironic
 
         Allows for the registration of Baremetal nodes with Ironic
@@ -175,16 +207,19 @@ class BaremetalCloudMixin:
         created are deleted, and the node is removed from Ironic.
 
         :param nics:
-            An array of MAC addresses that represent the
-            network interfaces for the node to be created.
+            An array of ports that represent the network interfaces for the
+            node to be created. The ports are created after the node is
+            enrolled but before it goes through cleaning.
 
             Example::
 
                 [
-                    {'mac': 'aa:bb:cc:dd:ee:01'},
-                    {'mac': 'aa:bb:cc:dd:ee:02'}
+                    {'address': 'aa:bb:cc:dd:ee:01'},
+                    {'address': 'aa:bb:cc:dd:ee:02'}
                 ]
 
+            Alternatively, you can provide an array of MAC addresses.
+
         :param wait: Boolean value, defaulting to false, to wait for the node
             to reach the available state where the node can be provisioned. It
             must be noted, when set to false, the method will still wait for
@@ -196,131 +231,69 @@ class BaremetalCloudMixin:
         :param lock_timeout: Integer value, defaulting to 600 seconds, for
             locks to clear.
 
+        :param provision_state: The expected provision state, one of "enroll"
+            "manageable" or "available". Using "available" results in automated
+            cleaning.
+
         :param kwargs: Key value pairs to be passed to the Ironic API,
-            including uuid, name, chassis_uuid, driver_info, parameters.
+            including uuid, name, chassis_uuid, driver_info, properties.
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: Returns a ``munch.Munch`` representing the new
-            baremetal node.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: Current state of the node.
         """
+        if provision_state not in ('enroll', 'manageable', 'available'):
+            raise ValueError(
+                'Initial provision state must be enroll, '
+                'manageable or available, got %s' % provision_state
+            )
 
-        msg = ("Baremetal machine node failed to be created.")
-        port_msg = ("Baremetal machine port failed to be created.")
+        # Available is tricky: it cannot be directly requested on newer API
+        # versions, we need to go through cleaning. But we cannot go through
+        # cleaning until we create ports.
+        if provision_state != 'available':
+            kwargs['provision_state'] = 'enroll'
+        machine = self.baremetal.create_node(**kwargs)
 
-        url = '/nodes'
-        # TODO(TheJulia): At some point we need to figure out how to
-        # handle data across when the requestor is defining newer items
-        # with the older api.
-        machine = self._baremetal_client.post(url,
-                                              json=kwargs,
-                                              error_message=msg,
-                                              microversion="1.6")
+        with self._delete_node_on_error(machine):
+            # Making a node at least manageable
+            if (
+                machine.provision_state == 'enroll'
+                and provision_state != 'enroll'
+            ):
+                machine = self.baremetal.set_node_provision_state(
+                    machine, 'manage', wait=True, timeout=timeout
+                )
+                machine = self.baremetal.wait_for_node_reservation(
+                    machine, timeout=lock_timeout
+                )
 
-        created_nics = []
-        try:
-            for row in nics:
-                payload = {'address': row['mac'],
-                           'node_uuid': machine['uuid']}
-                nic = self._baremetal_client.post('/ports',
-                                                  json=payload,
-                                                  error_message=port_msg)
-                created_nics.append(nic['uuid'])
-
-        except Exception as e:
-            self.log.debug("ironic NIC registration failed", exc_info=True)
-            # TODO(mordred) Handle failures here
+            # Create NICs before trying to run cleaning
+            created_nics = []
             try:
+                for port in _normalize_port_list(nics):
+                    nic = self.baremetal.create_port(
+                        node_id=machine.id, **port
+                    )
+                    created_nics.append(nic.id)
+
+            except Exception:
                 for uuid in created_nics:
                     try:
-                        port_url = '/ports/{uuid}'.format(uuid=uuid)
-                        # NOTE(TheJulia): Added in hope that it is logged.
-                        port_msg = ('Failed to delete port {port} for node '
-                                    '{node}').format(port=uuid,
-                                                     node=machine['uuid'])
-                        self._baremetal_client.delete(port_url,
-                                                      error_message=port_msg)
+                        self.baremetal.delete_port(uuid)
                     except Exception:
                         pass
-            finally:
-                version = "1.6"
-                msg = "Baremetal machine failed to be deleted."
-                url = '/nodes/{node_id}'.format(
-                    node_id=machine['uuid'])
-                self._baremetal_client.delete(url,
-                                              error_message=msg,
-                                              microversion=version)
-            raise exc.OpenStackCloudException(
-                "Error registering NICs with the baremetal service: %s"
-                % str(e))
+                raise
 
-        with _utils.shade_exceptions(
-                "Error transitioning node to available state"):
-            if wait:
-                for count in utils.iterate_timeout(
-                        timeout,
-                        "Timeout waiting for node transition to "
-                        "available state"):
+            if (
+                machine.provision_state != 'available'
+                and provision_state == 'available'
+            ):
+                machine = self.baremetal.set_node_provision_state(
+                    machine, 'provide', wait=wait, timeout=timeout
+                )
 
-                    machine = self.get_machine(machine['uuid'])
-
-                    # Note(TheJulia): Per the Ironic state code, a node
-                    # that fails returns to enroll state, which means a failed
-                    # node cannot be determined at this point in time.
-                    if machine['provision_state'] in ['enroll']:
-                        self.node_set_provision_state(
-                            machine['uuid'], 'manage')
-                    elif machine['provision_state'] in ['manageable']:
-                        self.node_set_provision_state(
-                            machine['uuid'], 'provide')
-                    elif machine['last_error'] is not None:
-                        raise exc.OpenStackCloudException(
-                            "Machine encountered a failure: %s"
-                            % machine['last_error'])
-
-                    # Note(TheJulia): Earlier versions of Ironic default to
-                    # None and later versions default to available up until
-                    # the introduction of enroll state.
-                    # Note(TheJulia): The node will transition through
-                    # cleaning if it is enabled, and we will wait for
-                    # completion.
-                    elif machine['provision_state'] in ['available', None]:
-                        break
-
-            else:
-                if machine['provision_state'] in ['enroll']:
-                    self.node_set_provision_state(machine['uuid'], 'manage')
-                    # Note(TheJulia): We need to wait for the lock to clear
-                    # before we attempt to set the machine into provide state
-                    # which allows for the transition to available.
-                    for count in utils.iterate_timeout(
-                            lock_timeout,
-                            "Timeout waiting for reservation to clear "
-                            "before setting provide state"):
-                        machine = self.get_machine(machine['uuid'])
-                        if (machine['reservation'] is None
-                                and machine['provision_state'] != 'enroll'):
-                            # NOTE(TheJulia): In this case, the node has
-                            # has moved on from the previous state and is
-                            # likely not being verified, as no lock is
-                            # present on the node.
-                            self.node_set_provision_state(
-                                machine['uuid'], 'provide')
-                            machine = self.get_machine(machine['uuid'])
-                            break
-
-                        elif machine['provision_state'] in [
-                                'cleaning',
-                                'available']:
-                            break
-
-                        elif machine['last_error'] is not None:
-                            raise exc.OpenStackCloudException(
-                                "Machine encountered a failure: %s"
-                                % machine['last_error'])
-        if not isinstance(machine, str):
-            return self._normalize_machine(machine)
-        else:
             return machine
 
     def unregister_machine(self, nics, uuid, wait=None, timeout=600):
@@ -342,15 +315,18 @@ class BaremetalCloudMixin:
         :raises: OpenStackCloudException on operation failure.
         """
         if wait is not None:
-            warnings.warn("wait argument is deprecated and has no effect",
-                          DeprecationWarning)
+            warnings.warn(
+                "wait argument is deprecated and has no effect",
+                DeprecationWarning,
+            )
 
         machine = self.get_machine(uuid)
         invalid_states = ['active', 'cleaning', 'clean wait', 'clean failed']
         if machine['provision_state'] in invalid_states:
             raise exc.OpenStackCloudException(
                 "Error unregistering node '%s' due to current provision "
-                "state '%s'" % (uuid, machine['provision_state']))
+                "state '%s'" % (uuid, machine['provision_state'])
+            )
 
         # NOTE(TheJulia) There is a high possibility of a lock being present
         # if the machine was just moved through the state machine. This was
@@ -361,11 +337,12 @@ class BaremetalCloudMixin:
         except exc.OpenStackCloudException as e:
             raise exc.OpenStackCloudException(
                 "Error unregistering node '%s': Exception occured while"
-                " waiting to be able to proceed: %s" % (machine['uuid'], e))
+                " waiting to be able to proceed: %s" % (machine['uuid'], e)
+            )
 
-        for nic in nics:
+        for nic in _normalize_port_list(nics):
             try:
-                port = next(self.baremetal.ports(address=nic['mac']))
+                port = next(self.baremetal.ports(address=nic['address']))
             except StopIteration:
                 continue
             self.baremetal.delete_port(port.id)
@@ -404,10 +381,10 @@ class BaremetalCloudMixin:
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: ``munch.Munch`` representing the newly updated node.
+        :returns: Current state of the node.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
         """
-        return self._normalize_machine(
-            self.baremetal.patch_node(name_or_id, patch))
+        return self.baremetal.patch_node(name_or_id, patch)
 
     def update_machine(self, name_or_id, **attrs):
         """Update a machine with new configuration information
@@ -420,7 +397,7 @@ class BaremetalCloudMixin:
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: ``munch.Munch`` containing a machine sub-dictonary consisting
+        :returns: Dictionary containing a machine sub-dictonary consisting
             of the updated data returned from the API update operation, and a
             list named changes which contains all of the API paths that
             received updates.
@@ -428,30 +405,28 @@ class BaremetalCloudMixin:
         machine = self.get_machine(name_or_id)
         if not machine:
             raise exc.OpenStackCloudException(
-                "Machine update failed to find Machine: %s. " % name_or_id)
+                "Machine update failed to find Machine: %s. " % name_or_id
+            )
 
-        new_config = dict(machine, **attrs)
+        new_config = dict(machine._to_munch(), **attrs)
 
         try:
-            patch = jsonpatch.JsonPatch.from_diff(machine, new_config)
+            patch = jsonpatch.JsonPatch.from_diff(
+                machine._to_munch(), new_config
+            )
         except Exception as e:
             raise exc.OpenStackCloudException(
                 "Machine update failed - Error generating JSON patch object "
                 "for submission to the API. Machine: %s Error: %s"
-                % (name_or_id, e))
+                % (name_or_id, e)
+            )
 
         if not patch:
-            return dict(
-                node=machine,
-                changes=None
-            )
+            return dict(node=machine, changes=None)
 
         change_list = [change['path'] for change in patch]
         node = self.baremetal.update_node(machine, **attrs)
-        return dict(
-            node=self._normalize_machine(node),
-            changes=change_list
-        )
+        return dict(node=node, changes=change_list)
 
     def attach_port_to_machine(self, name_or_id, port_name_or_id):
         """Attach a virtual port to the bare metal machine.
@@ -481,7 +456,8 @@ class BaremetalCloudMixin:
         """List virtual ports attached to the bare metal machine.
 
         :param string name_or_id: A machine name or UUID.
-        :returns: List of ``munch.Munch`` representing the ports.
+        :returns: List of ``openstack.Resource`` objects representing
+            the ports.
         """
         machine = self.get_machine(name_or_id)
         vif_ids = self.baremetal.list_node_vifs(machine)
@@ -503,16 +479,16 @@ class BaremetalCloudMixin:
         self.baremetal.validate_node(name_or_id, required=ifaces)
 
     def validate_node(self, uuid):
-        warnings.warn('validate_node is deprecated, please use '
-                      'validate_machine instead', DeprecationWarning)
+        warnings.warn(
+            'validate_node is deprecated, please use '
+            'validate_machine instead',
+            DeprecationWarning,
+        )
         self.baremetal.validate_node(uuid)
 
-    def node_set_provision_state(self,
-                                 name_or_id,
-                                 state,
-                                 configdrive=None,
-                                 wait=False,
-                                 timeout=3600):
+    def node_set_provision_state(
+        self, name_or_id, state, configdrive=None, wait=False, timeout=3600
+    ):
         """Set Node Provision State
 
         Enables a user to provision a Machine and optionally define a
@@ -535,19 +511,21 @@ class BaremetalCloudMixin:
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: ``munch.Munch`` representing the current state of the machine
-            upon exit of the method.
+        :returns: Current state of the machine upon exit of the method.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
         """
         node = self.baremetal.set_node_provision_state(
-            name_or_id, target=state, config_drive=configdrive,
-            wait=wait, timeout=timeout)
-        return self._normalize_machine(node)
+            name_or_id,
+            target=state,
+            config_drive=configdrive,
+            wait=wait,
+            timeout=timeout,
+        )
+        return node
 
     def set_machine_maintenance_state(
-            self,
-            name_or_id,
-            state=True,
-            reason=None):
+        self, name_or_id, state=True, reason=None
+    ):
         """Set Baremetal Machine Maintenance State
 
         Sets Baremetal maintenance state and maintenance reason.
@@ -631,28 +609,33 @@ class BaremetalCloudMixin:
         """
         self.baremetal.set_node_power_state(name_or_id, 'rebooting')
 
-    def activate_node(self, uuid, configdrive=None,
-                      wait=False, timeout=1200):
+    def activate_node(self, uuid, configdrive=None, wait=False, timeout=1200):
         self.node_set_provision_state(
-            uuid, 'active', configdrive, wait=wait, timeout=timeout)
+            uuid, 'active', configdrive, wait=wait, timeout=timeout
+        )
 
-    def deactivate_node(self, uuid, wait=False,
-                        timeout=1200):
+    def deactivate_node(self, uuid, wait=False, timeout=1200):
         self.node_set_provision_state(
-            uuid, 'deleted', wait=wait, timeout=timeout)
+            uuid, 'deleted', wait=wait, timeout=timeout
+        )
 
     def set_node_instance_info(self, uuid, patch):
-        warnings.warn("The set_node_instance_info call is deprecated, "
-                      "use patch_machine or update_machine instead",
-                      DeprecationWarning)
+        warnings.warn(
+            "The set_node_instance_info call is deprecated, "
+            "use patch_machine or update_machine instead",
+            DeprecationWarning,
+        )
         return self.patch_machine(uuid, patch)
 
     def purge_node_instance_info(self, uuid):
-        warnings.warn("The purge_node_instance_info call is deprecated, "
-                      "use patch_machine or update_machine instead",
-                      DeprecationWarning)
-        return self.patch_machine(uuid,
-                                  dict(path='/instance_info', op='remove'))
+        warnings.warn(
+            "The purge_node_instance_info call is deprecated, "
+            "use patch_machine or update_machine instead",
+            DeprecationWarning,
+        )
+        return self.patch_machine(
+            uuid, dict(path='/instance_info', op='remove')
+        )
 
     def wait_for_baremetal_node_lock(self, node, timeout=30):
         """Wait for a baremetal node to have no lock.
@@ -662,7 +645,10 @@ class BaremetalCloudMixin:
         :raises: OpenStackCloudException upon client failure.
         :returns: None
         """
-        warnings.warn("The wait_for_baremetal_node_lock call is deprecated "
-                      "in favor of wait_for_node_reservation on the baremetal "
-                      "proxy", DeprecationWarning)
+        warnings.warn(
+            "The wait_for_baremetal_node_lock call is deprecated "
+            "in favor of wait_for_node_reservation on the baremetal "
+            "proxy",
+            DeprecationWarning,
+        )
         self.baremetal.wait_for_node_reservation(node, timeout)
