@@ -16,14 +16,16 @@
 # import jsonpatch
 import types  # noqa
 
-from openstack.cloud import exc
-from openstack.cloud import _normalize
 from openstack.cloud import _utils
+from openstack.cloud import exc
 from openstack import exceptions
+from openstack.network.v2._proxy import Proxy
 from openstack import proxy
+from openstack import utils
 
 
-class SecurityGroupCloudMixin(_normalize.Normalizer):
+class SecurityGroupCloudMixin:
+    network: Proxy
 
     def __init__(self):
         self.secgroup_source = self.config.config['secgroup_source']
@@ -39,7 +41,8 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         """List all available security groups.
 
         :param filters: (optional) dict of filter conditions to push down
-        :returns: A list of security group ``munch.Munch``.
+        :returns: A list of security group
+            ``openstack.network.v2.security_group.SecurityGroup``.
 
         """
         # Security groups not supported
@@ -56,16 +59,16 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         if self._use_neutron_secgroups():
             # pass filters dict to the list to filter as much as possible on
             # the server side
-            return list(
-                self.network.security_groups(allow_unknown_params=True,
-                                             **filters))
+            return list(self.network.security_groups(**filters))
 
         # Handle nova security groups
         else:
-            data = proxy._json_response(self.compute.get(
-                '/os-security-groups', params=filters))
+            data = proxy._json_response(
+                self.compute.get('/os-security-groups', params=filters)
+            )
         return self._normalize_secgroups(
-            self._get_and_munchify('security_groups', data))
+            self._get_and_munchify('security_groups', data)
+        )
 
     def get_security_group(self, name_or_id, filters=None):
         """Get a security group by name or ID.
@@ -76,46 +79,51 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
             of this dictionary may, themselves, be dictionaries. Example::
 
                 {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
+                    'last_name': 'Smith',
+                    'other': {
+                        'gender': 'Female'
+                    }
                 }
 
             OR
             A string containing a jmespath expression for further filtering.
             Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"
 
-        :returns: A security group ``munch.Munch`` or None if no matching
-                  security group is found.
+        :returns: A security group
+            ``openstack.network.v2.security_group.SecurityGroup``
+            or None if no matching security group is found.
 
         """
-        return _utils._get_entity(
-            self, 'security_group', name_or_id, filters)
+        return _utils._get_entity(self, 'security_group', name_or_id, filters)
 
     def get_security_group_by_id(self, id):
-        """ Get a security group by ID
+        """Get a security group by ID
 
         :param id: ID of the security group.
-        :returns: A security group ``munch.Munch``.
+        :returns: A security group
+            ``openstack.network.v2.security_group.SecurityGroup``.
         """
         if not self._has_secgroups():
             raise exc.OpenStackCloudUnavailableFeature(
                 "Unavailable feature: security groups"
             )
-        error_message = ("Error getting security group with"
-                         " ID {id}".format(id=id))
+        error_message = "Error getting security group with" " ID {id}".format(
+            id=id
+        )
         if self._use_neutron_secgroups():
             return self.network.get_security_group(id)
         else:
             data = proxy._json_response(
-                self.compute.get(
-                    '/os-security-groups/{id}'.format(id=id)),
-                error_message=error_message)
+                self.compute.get('/os-security-groups/{id}'.format(id=id)),
+                error_message=error_message,
+            )
         return self._normalize_secgroup(
-            self._get_and_munchify('security_group', data))
+            self._get_and_munchify('security_group', data)
+        )
 
-    def create_security_group(self, name, description, project_id=None):
+    def create_security_group(
+        self, name, description, project_id=None, stateful=None
+    ):
         """Create a new security group
 
         :param string name: A name for the security group.
@@ -123,12 +131,14 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         :param string project_id:
             Specify the project ID this security group will be created
             on (admin-only).
+        :param string stateful: Whether the security group is stateful or not.
 
-        :returns: A ``munch.Munch`` representing the new security group.
+        :returns: A ``openstack.network.v2.security_group.SecurityGroup``
+            representing the new security group.
 
         :raises: OpenStackCloudException on operation error.
         :raises: OpenStackCloudUnavailableFeature if security groups are
-                 not supported on this cloud.
+            not supported on this cloud.
         """
 
         # Security groups not supported
@@ -138,20 +148,23 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
             )
 
         data = []
-        security_group_json = {
-            'name': name, 'description': description
-        }
+        security_group_json = {'name': name, 'description': description}
+        if stateful is not None:
+            security_group_json['stateful'] = stateful
         if project_id is not None:
             security_group_json['tenant_id'] = project_id
         if self._use_neutron_secgroups():
-            return self.network.create_security_group(
-                **security_group_json)
+            return self.network.create_security_group(**security_group_json)
         else:
-            data = proxy._json_response(self.compute.post(
-                '/os-security-groups',
-                json={'security_group': security_group_json}))
+            data = proxy._json_response(
+                self.compute.post(
+                    '/os-security-groups',
+                    json={'security_group': security_group_json},
+                )
+            )
         return self._normalize_secgroup(
-            self._get_and_munchify('security_group', data))
+            self._get_and_munchify('security_group', data)
+        )
 
     def delete_security_group(self, name_or_id):
         """Delete a security group
@@ -162,7 +175,7 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         :raises: OpenStackCloudUnavailableFeature if security groups are
-                 not supported on this cloud.
+            not supported on this cloud.
         """
         # Security groups not supported
         if not self._has_secgroups():
@@ -174,21 +187,26 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         #                the delete.
         secgroup = self.get_security_group(name_or_id)
         if secgroup is None:
-            self.log.debug('Security group %s not found for deleting',
-                           name_or_id)
+            self.log.debug(
+                'Security group %s not found for deleting', name_or_id
+            )
             return False
 
         if self._use_neutron_secgroups():
             self.network.delete_security_group(
-                secgroup['id'], ignore_missing=False)
+                secgroup['id'], ignore_missing=False
+            )
             return True
 
         else:
-            proxy._json_response(self.compute.delete(
-                '/os-security-groups/{id}'.format(id=secgroup['id'])))
+            proxy._json_response(
+                self.compute.delete(
+                    '/os-security-groups/{id}'.format(id=secgroup['id'])
+                )
+            )
             return True
 
-    @_utils.valid_kwargs('name', 'description')
+    @_utils.valid_kwargs('name', 'description', 'stateful')
     def update_security_group(self, name_or_id, **kwargs):
         """Update a security group
 
@@ -196,7 +214,8 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         :param string name: New name for the security group.
         :param string description: New description for the security group.
 
-        :returns: A ``munch.Munch`` describing the updated security group.
+        :returns: A ``openstack.network.v2.security_group.SecurityGroup``
+            describing the updated security group.
 
         :raises: OpenStackCloudException on operation error.
         """
@@ -210,33 +229,38 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
 
         if group is None:
             raise exc.OpenStackCloudException(
-                "Security group %s not found." % name_or_id)
+                "Security group %s not found." % name_or_id
+            )
 
         if self._use_neutron_secgroups():
-            return self.network.update_security_group(
-                group['id'],
-                **kwargs
-            )
+            return self.network.update_security_group(group['id'], **kwargs)
         else:
             for key in ('name', 'description'):
                 kwargs.setdefault(key, group[key])
             data = proxy._json_response(
                 self.compute.put(
                     '/os-security-groups/{id}'.format(id=group['id']),
-                    json={'security_group': kwargs}))
+                    json={'security_group': kwargs},
+                )
+            )
         return self._normalize_secgroup(
-            self._get_and_munchify('security_group', data))
+            self._get_and_munchify('security_group', data)
+        )
 
-    def create_security_group_rule(self,
-                                   secgroup_name_or_id,
-                                   port_range_min=None,
-                                   port_range_max=None,
-                                   protocol=None,
-                                   remote_ip_prefix=None,
-                                   remote_group_id=None,
-                                   direction='ingress',
-                                   ethertype='IPv4',
-                                   project_id=None):
+    def create_security_group_rule(
+        self,
+        secgroup_name_or_id,
+        port_range_min=None,
+        port_range_max=None,
+        protocol=None,
+        remote_ip_prefix=None,
+        remote_group_id=None,
+        remote_address_group_id=None,
+        direction='ingress',
+        ethertype='IPv4',
+        project_id=None,
+        description=None,
+    ):
         """Create a new security group rule
 
         :param string secgroup_name_or_id:
@@ -265,6 +289,9 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         :param string remote_group_id:
             The remote group ID to be associated with this security group
             rule.
+        :param string remote_address_group_id:
+            The remote address group ID to be associated with this security
+            group rule.
         :param string direction:
             Ingress or egress: The direction in which the security group
             rule is applied. For a compute instance, an ingress security
@@ -277,8 +304,10 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         :param string project_id:
             Specify the project ID this security group will be created
             on (admin-only).
-
-        :returns: A ``munch.Munch`` representing the new security group rule.
+        :param string description:
+            Description of the rule, max 255 characters.
+        :returns: A ``openstack.network.v2.security_group.SecurityGroup``
+            representing the new security group rule.
 
         :raises: OpenStackCloudException on operation error.
         """
@@ -291,30 +320,32 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
         secgroup = self.get_security_group(secgroup_name_or_id)
         if not secgroup:
             raise exc.OpenStackCloudException(
-                "Security group %s not found." % secgroup_name_or_id)
+                "Security group %s not found." % secgroup_name_or_id
+            )
 
         if self._use_neutron_secgroups():
             # NOTE: Nova accepts -1 port numbers, but Neutron accepts None
             # as the equivalent value.
             rule_def = {
                 'security_group_id': secgroup['id'],
-                'port_range_min':
-                    None if port_range_min == -1 else port_range_min,
-                'port_range_max':
-                    None if port_range_max == -1 else port_range_max,
+                'port_range_min': None
+                if port_range_min == -1
+                else port_range_min,
+                'port_range_max': None
+                if port_range_max == -1
+                else port_range_max,
                 'protocol': protocol,
                 'remote_ip_prefix': remote_ip_prefix,
                 'remote_group_id': remote_group_id,
+                'remote_address_group_id': remote_address_group_id,
                 'direction': direction,
-                'ethertype': ethertype
+                'ethertype': ethertype,
             }
             if project_id is not None:
-                rule_def['project_id'] = project_id
                 rule_def['tenant_id'] = project_id
-
-            return self.network.create_security_group_rule(
-                **rule_def
-            )
+            if description is not None:
+                rule_def["description"] = description
+            return self.network.create_security_group_rule(**rule_def)
         else:
             # NOTE: Neutron accepts None for protocol. Nova does not.
             if protocol is None:
@@ -325,7 +356,8 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
                     'Rule creation failed: Nova does not support egress rules'
                 )
                 raise exc.OpenStackCloudException(
-                    'No support for egress rules')
+                    'No support for egress rules'
+                )
 
             # NOTE: Neutron accepts None for ports, but Nova requires -1
             # as the equivalent value for ICMP.
@@ -345,24 +377,28 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
                     port_range_min = 1
                     port_range_max = 65535
 
-            security_group_rule_dict = dict(security_group_rule=dict(
-                parent_group_id=secgroup['id'],
-                ip_protocol=protocol,
-                from_port=port_range_min,
-                to_port=port_range_max,
-                cidr=remote_ip_prefix,
-                group_id=remote_group_id
-            ))
+            security_group_rule_dict = dict(
+                security_group_rule=dict(
+                    parent_group_id=secgroup['id'],
+                    ip_protocol=protocol,
+                    from_port=port_range_min,
+                    to_port=port_range_max,
+                    cidr=remote_ip_prefix,
+                    group_id=remote_group_id,
+                )
+            )
             if project_id is not None:
-                security_group_rule_dict[
-                    'security_group_rule']['tenant_id'] = project_id
+                security_group_rule_dict['security_group_rule'][
+                    'tenant_id'
+                ] = project_id
             data = proxy._json_response(
                 self.compute.post(
-                    '/os-security-group-rules',
-                    json=security_group_rule_dict
-                ))
+                    '/os-security-group-rules', json=security_group_rule_dict
+                )
+            )
         return self._normalize_secgroup_rule(
-            self._get_and_munchify('security_group_rule', data))
+            self._get_and_munchify('security_group_rule', data)
+        )
 
     def delete_security_group_rule(self, rule_id):
         """Delete a security group rule
@@ -373,7 +409,7 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         :raises: OpenStackCloudUnavailableFeature if security groups are
-                 not supported on this cloud.
+            not supported on this cloud.
         """
         # Security groups not supported
         if not self._has_secgroups():
@@ -383,8 +419,7 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
 
         if self._use_neutron_secgroups():
             self.network.delete_security_group_rule(
-                rule_id,
-                ignore_missing=False
+                rule_id, ignore_missing=False
             )
             return True
 
@@ -392,7 +427,9 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
             try:
                 exceptions.raise_from_response(
                     self.compute.delete(
-                        '/os-security-group-rules/{id}'.format(id=rule_id)))
+                        '/os-security-group-rules/{id}'.format(id=rule_id)
+                    )
+                )
             except exc.OpenStackCloudResourceNotFound:
                 return False
 
@@ -405,5 +442,128 @@ class SecurityGroupCloudMixin(_normalize.Normalizer):
             return self.secgroup_source.lower() in ('nova', 'neutron')
 
     def _use_neutron_secgroups(self):
-        return (self.has_service('network')
-                and self.secgroup_source == 'neutron')
+        return (
+            self.has_service('network') and self.secgroup_source == 'neutron'
+        )
+
+    def _normalize_secgroups(self, groups):
+        """Normalize the structure of security groups
+
+        This makes security group dicts, as returned from nova, look like the
+        security group dicts as returned from neutron. This does not make them
+        look exactly the same, but it's pretty close.
+
+        :param list groups: A list of security group dicts.
+
+        :returns: A list of normalized dicts.
+        """
+        ret = []
+        for group in groups:
+            ret.append(self._normalize_secgroup(group))
+        return ret
+
+    # TODO(stephenfin): Remove this once we get rid of support for nova
+    # secgroups
+    def _normalize_secgroup(self, group):
+        ret = utils.Munch()
+        # Copy incoming group because of shared dicts in unittests
+        group = group.copy()
+
+        # Discard noise
+        self._remove_novaclient_artifacts(group)
+
+        rules = self._normalize_secgroup_rules(
+            group.pop('security_group_rules', group.pop('rules', []))
+        )
+        project_id = group.pop('tenant_id', '')
+        project_id = group.pop('project_id', project_id)
+
+        ret['location'] = self._get_current_location(project_id=project_id)
+        ret['id'] = group.pop('id')
+        ret['name'] = group.pop('name')
+        ret['security_group_rules'] = rules
+        ret['description'] = group.pop('description')
+        ret['properties'] = group
+
+        if self._use_neutron_secgroups():
+            ret['stateful'] = group.pop('stateful', True)
+
+        # Backwards compat with Neutron
+        if not self.strict_mode:
+            ret['tenant_id'] = project_id
+            ret['project_id'] = project_id
+            for key, val in ret['properties'].items():
+                ret.setdefault(key, val)
+
+        return ret
+
+    # TODO(stephenfin): Remove this once we get rid of support for nova
+    # secgroups
+    def _normalize_secgroup_rules(self, rules):
+        """Normalize the structure of nova security group rules
+
+        Note that nova uses -1 for non-specific port values, but neutron
+        represents these with None.
+
+        :param list rules: A list of security group rule dicts.
+
+        :returns: A list of normalized dicts.
+        """
+        ret = []
+        for rule in rules:
+            ret.append(self._normalize_secgroup_rule(rule))
+        return ret
+
+    # TODO(stephenfin): Remove this once we get rid of support for nova
+    # secgroups
+    def _normalize_secgroup_rule(self, rule):
+        ret = utils.Munch()
+        # Copy incoming rule because of shared dicts in unittests
+        rule = rule.copy()
+
+        ret['id'] = rule.pop('id')
+        ret['direction'] = rule.pop('direction', 'ingress')
+        ret['ethertype'] = rule.pop('ethertype', 'IPv4')
+        port_range_min = rule.get(
+            'port_range_min', rule.pop('from_port', None)
+        )
+        if port_range_min == -1:
+            port_range_min = None
+        if port_range_min is not None:
+            port_range_min = int(port_range_min)
+        ret['port_range_min'] = port_range_min
+        port_range_max = rule.pop('port_range_max', rule.pop('to_port', None))
+        if port_range_max == -1:
+            port_range_max = None
+        if port_range_min is not None:
+            port_range_min = int(port_range_min)
+        ret['port_range_max'] = port_range_max
+        ret['protocol'] = rule.pop('protocol', rule.pop('ip_protocol', None))
+        ret['remote_ip_prefix'] = rule.pop(
+            'remote_ip_prefix', rule.pop('ip_range', {}).get('cidr', None)
+        )
+        ret['security_group_id'] = rule.pop(
+            'security_group_id', rule.pop('parent_group_id', None)
+        )
+        ret['remote_group_id'] = rule.pop('remote_group_id', None)
+        project_id = rule.pop('tenant_id', '')
+        project_id = rule.pop('project_id', project_id)
+        ret['location'] = self._get_current_location(project_id=project_id)
+        ret['properties'] = rule
+
+        # Backwards compat with Neutron
+        if not self.strict_mode:
+            ret['tenant_id'] = project_id
+            ret['project_id'] = project_id
+            for key, val in ret['properties'].items():
+                ret.setdefault(key, val)
+        return ret
+
+    def _remove_novaclient_artifacts(self, item):
+        # Remove novaclient artifacts
+        item.pop('links', None)
+        item.pop('NAME_ATTR', None)
+        item.pop('HUMAN_ID', None)
+        item.pop('human_id', None)
+        item.pop('request_ids', None)
+        item.pop('x_openstack_request_ids', None)
