@@ -22,13 +22,10 @@ import uuid
 
 from decorator import decorator
 import jmespath
-import munch
 import netifaces
-import sre_constants
 
 from openstack import _log
 from openstack.cloud import exc
-from openstack.cloud import meta
 
 
 _decorated_methods = []
@@ -95,7 +92,7 @@ def _filter_list(data, name_or_id, filters):
         bad_pattern = False
         try:
             fn_reg = re.compile(fnmatch.translate(name_or_id))
-        except sre_constants.error:
+        except re.error:
             # If the fnmatch re doesn't compile, then we don't care,
             # but log it in case the user DID pass a pattern but did
             # it poorly and wants to know what went wrong with their
@@ -105,8 +102,9 @@ def _filter_list(data, name_or_id, filters):
             e_id = _make_unicode(e.get('id', None))
             e_name = _make_unicode(e.get('name', None))
 
-            if ((e_id and e_id == name_or_id)
-                    or (e_name and e_name == name_or_id)):
+            if (e_id and e_id == name_or_id) or (
+                e_name and e_name == name_or_id
+            ):
                 identifier_matches.append(e)
             else:
                 # Only try fnmatch if we don't match exactly
@@ -115,8 +113,9 @@ def _filter_list(data, name_or_id, filters):
                     # so that we log the bad pattern
                     bad_pattern = True
                     continue
-                if ((e_id and fn_reg.match(e_id))
-                        or (e_name and fn_reg.match(e_name))):
+                if (e_id and fn_reg.match(e_id)) or (
+                    e_name and fn_reg.match(e_name)
+                ):
                     identifier_matches.append(e)
         if not identifier_matches and bad_pattern:
             log.debug("Bad pattern passed to fnmatch", exc_info=True)
@@ -175,8 +174,9 @@ def _get_entity(cloud, resource, name_or_id, filters, **kwargs):
     # an additional call, it's simple enough to test to see if we got an
     # object and just short-circuit return it.
 
-    if (hasattr(name_or_id, 'id')
-            or (isinstance(name_or_id, dict) and 'id' in name_or_id)):
+    if hasattr(name_or_id, 'id') or (
+        isinstance(name_or_id, dict) and 'id' in name_or_id
+    ):
         return name_or_id
 
     # If a uuid is passed short-circuit it calling the
@@ -186,43 +186,22 @@ def _get_entity(cloud, resource, name_or_id, filters, **kwargs):
         if get_resource:
             return get_resource(name_or_id)
 
-    search = resource if callable(resource) else getattr(
-        cloud, 'search_%ss' % resource, None)
+    search = (
+        resource
+        if callable(resource)
+        else getattr(cloud, 'search_%ss' % resource, None)
+    )
     if search:
         entities = search(name_or_id, filters, **kwargs)
         if entities:
             if len(entities) > 1:
                 raise exc.OpenStackCloudException(
-                    "Multiple matches found for %s" % name_or_id)
+                    "Multiple matches found for %s" % name_or_id
+                )
             return entities[0]
     return None
 
 
-def normalize_keystone_services(services):
-    """Normalize the structure of keystone services
-
-    In keystone v2, there is a field called "service_type". In v3, it's
-    "type". Just make the returned dict have both.
-
-    :param list services: A list of keystone service dicts
-
-    :returns: A list of normalized dicts.
-    """
-    ret = []
-    for service in services:
-        service_type = service.get('type', service.get('service_type'))
-        new_service = {
-            'id': service['id'],
-            'name': service['name'],
-            'description': service.get('description', None),
-            'type': service_type,
-            'service_type': service_type,
-            'enabled': service['enabled']
-        }
-        ret.append(new_service)
-    return meta.obj_list_to_munch(ret)
-
-
 def localhost_supports_ipv6():
     """Determine whether the local host supports IPv6
 
@@ -237,109 +216,6 @@ def localhost_supports_ipv6():
         return False
 
 
-def normalize_users(users):
-    ret = [
-        dict(
-            id=user.get('id'),
-            email=user.get('email'),
-            name=user.get('name'),
-            username=user.get('username'),
-            default_project_id=user.get('default_project_id',
-                                        user.get('tenantId')),
-            domain_id=user.get('domain_id'),
-            enabled=user.get('enabled'),
-            description=user.get('description')
-        ) for user in users
-    ]
-    return meta.obj_list_to_munch(ret)
-
-
-def normalize_domains(domains):
-    ret = [
-        dict(
-            id=domain.get('id'),
-            name=domain.get('name'),
-            description=domain.get('description'),
-            enabled=domain.get('enabled'),
-        ) for domain in domains
-    ]
-    return meta.obj_list_to_munch(ret)
-
-
-def normalize_groups(domains):
-    """Normalize Identity groups."""
-    ret = [
-        dict(
-            id=domain.get('id'),
-            name=domain.get('name'),
-            description=domain.get('description'),
-            domain_id=domain.get('domain_id'),
-        ) for domain in domains
-    ]
-    return meta.obj_list_to_munch(ret)
-
-
-def normalize_role_assignments(assignments):
-    """Put role_assignments into a form that works with search/get interface.
-
-    Role assignments have the structure::
-
-        [
-            {
-                "role": {
-                    "id": "--role-id--"
-                },
-                "scope": {
-                    "domain": {
-                        "id": "--domain-id--"
-                    }
-                },
-                "user": {
-                    "id": "--user-id--"
-                }
-            },
-        ]
-
-    Which is hard to work with in the rest of our interface. Map this to be::
-
-        [
-            {
-                "id": "--role-id--",
-                "domain": "--domain-id--",
-                "user": "--user-id--",
-            }
-        ]
-
-    Scope can be "domain" or "project" and "user" can also be "group".
-
-    :param list assignments: A list of dictionaries of role assignments.
-
-    :returns: A list of flattened/normalized role assignment dicts.
-    """
-    new_assignments = []
-    for assignment in assignments:
-        new_val = munch.Munch({'id': assignment['role']['id']})
-        for scope in ('project', 'domain'):
-            if scope in assignment['scope']:
-                new_val[scope] = assignment['scope'][scope]['id']
-        for assignee in ('user', 'group'):
-            if assignment[assignee]:
-                new_val[assignee] = assignment[assignee]['id']
-        new_assignments.append(new_val)
-    return new_assignments
-
-
-def normalize_flavor_accesses(flavor_accesses):
-    """Normalize Flavor access list."""
-    return [munch.Munch(
-        dict(
-            flavor_id=acl.get('flavor_id'),
-            project_id=acl.get('project_id') or acl.get('tenant_id'),
-        )
-    ) for acl in flavor_accesses
-    ]
-
-
 def valid_kwargs(*valid_args):
     # This decorator checks if argument passed as **kwargs to a function are
     # present in valid_args.
@@ -361,8 +237,10 @@ def valid_kwargs(*valid_args):
             if k not in argspec.args[1:] and k not in valid_args:
                 raise TypeError(
                     "{f}() got an unexpected keyword argument "
-                    "'{arg}'".format(f=inspect.stack()[1][3], arg=k))
+                    "'{arg}'".format(f=inspect.stack()[1][3], arg=k)
+                )
         return func(*args, **kwargs)
+
     return func_wrapper
 
 
@@ -375,6 +253,7 @@ def _func_wrap(f):
     @functools.wraps(f)
     def inner(*args, **kwargs):
         return f(*args, **kwargs)
+
     return inner
 
 
@@ -384,20 +263,23 @@ def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
     def _inner_cache_on_arguments(func):
         def _cache_decorator(obj, *args, **kwargs):
             the_method = obj._get_cache(_cache_name).cache_on_arguments(
-                *cache_on_args, **cache_on_kwargs)(
-                    _func_wrap(func.__get__(obj, type(obj))))
+                *cache_on_args, **cache_on_kwargs
+            )(_func_wrap(func.__get__(obj, type(obj))))
             return the_method(*args, **kwargs)
 
         def invalidate(obj, *args, **kwargs):
-            return obj._get_cache(
-                _cache_name).cache_on_arguments()(func).invalidate(
-                    *args, **kwargs)
+            return (
+                obj._get_cache(_cache_name)
+                .cache_on_arguments()(func)
+                .invalidate(*args, **kwargs)
+            )
 
         _cache_decorator.invalidate = invalidate
         _cache_decorator.func = func
         _decorated_methods.append(func.__name__)
 
         return _cache_decorator
+
     return _inner_cache_on_arguments
 
 
@@ -451,7 +333,8 @@ def safe_dict_min(key, data):
                 raise exc.OpenStackCloudException(
                     "Search for minimum value failed. "
                     "Value for {key} is not an integer: {value}".format(
-                        key=key, value=d[key])
+                        key=key, value=d[key]
+                    )
                 )
             if (min_value is None) or (val < min_value):
                 min_value = val
@@ -483,16 +366,17 @@ def safe_dict_max(key, data):
                 raise exc.OpenStackCloudException(
                     "Search for maximum value failed. "
                     "Value for {key} is not an integer: {value}".format(
-                        key=key, value=d[key])
+                        key=key, value=d[key]
+                    )
                 )
             if (max_value is None) or (val > max_value):
                 max_value = val
     return max_value
 
 
-def _call_client_and_retry(client, url, retry_on=None,
-                           call_retries=3, retry_wait=2,
-                           **kwargs):
+def _call_client_and_retry(
+    client, url, retry_on=None, call_retries=3, retry_wait=2, **kwargs
+):
     """Method to provide retry operations.
 
     Some APIs utilize HTTP errors on certain operations to indicate that
@@ -522,18 +406,17 @@ def _call_client_and_retry(client, url, retry_on=None,
         retry_on = [retry_on]
 
     count = 0
-    while (count < call_retries):
+    while count < call_retries:
         count += 1
         try:
             ret_val = client(url, **kwargs)
         except exc.OpenStackCloudHTTPError as e:
-            if (retry_on is not None
-                    and e.response.status_code in retry_on):
-                log.debug('Received retryable error %(err)s, waiting '
-                          '%(wait)s seconds to retry', {
-                              'err': e.response.status_code,
-                              'wait': retry_wait
-                          })
+            if retry_on is not None and e.response.status_code in retry_on:
+                log.debug(
+                    'Received retryable error %(err)s, waiting '
+                    '%(wait)s seconds to retry',
+                    {'err': e.response.status_code, 'wait': retry_wait},
+                )
                 time.sleep(retry_wait)
                 continue
             else:
@@ -615,7 +498,8 @@ def range_filter(data, key, range_exp):
     # If parsing the range fails, it must be a bad value.
     if val_range is None:
         raise exc.OpenStackCloudException(
-            "Invalid range value: {value}".format(value=range_exp))
+            "Invalid range value: {value}".format(value=range_exp)
+        )
 
     op = val_range[0]
     if op:
@@ -654,9 +538,7 @@ def generate_patches_from_kwargs(operation, **kwargs):
     """
     patches = []
     for k, v in kwargs.items():
-        patch = {'op': operation,
-                 'value': v,
-                 'path': '/%s' % k}
+        patch = {'op': operation, 'value': v, 'path': '/%s' % k}
         patches.append(patch)
     return sorted(patches)
 
@@ -699,11 +581,13 @@ class FileSegment:
 
 
 def _format_uuid_string(string):
-    return (string.replace('urn:', '')
-                  .replace('uuid:', '')
-                  .strip('{}')
-                  .replace('-', '')
-                  .lower())
+    return (
+        string.replace('urn:', '')
+        .replace('uuid:', '')
+        .strip('{}')
+        .replace('-', '')
+        .lower()
+    )
 
 
 def _is_uuid_like(val):
