@@ -12,6 +12,7 @@
 import copy
 import functools
 import queue
+
 # import types so that we can reference ListType in sphinx param declarations.
 # We can't just use list, because sphinx gets confused by
 # openstack.resource.Resource.list and openstack.resource2.Resource.list
@@ -21,7 +22,6 @@ import warnings
 import dogpile.cache
 import keystoneauth1.exceptions
 import keystoneauth1.session
-import munch
 import requests.models
 import requestsexceptions
 
@@ -33,6 +33,7 @@ from openstack.cloud import exc
 from openstack.cloud import meta
 import openstack.config
 from openstack.config import cloud_region as cloud_region_mod
+from openstack import exceptions
 from openstack import proxy
 from openstack import utils
 
@@ -60,6 +61,7 @@ class _OpenStackCloudMixin:
     :param bool strict: Only return documented attributes for each resource
                         as per the Data Model contract. (Default False)
     """
+
     _OBJECT_MD5_KEY = 'x-sdk-md5'
     _OBJECT_SHA256_KEY = 'x-sdk-sha256'
     _OBJECT_AUTOCREATE_KEY = 'x-sdk-autocreated'
@@ -73,7 +75,6 @@ class _OpenStackCloudMixin:
     _SHADE_OBJECT_AUTOCREATE_KEY = 'x-object-meta-x-shade-autocreated'
 
     def __init__(self):
-
         super(_OpenStackCloudMixin, self).__init__()
 
         self.log = _log.setup_logging('openstack')
@@ -90,7 +91,8 @@ class _OpenStackCloudMixin:
         # cert verification
         if not self.verify:
             self.log.debug(
-                "Turning off Insecure SSL warnings since verify=False")
+                "Turning off Insecure SSL warnings since verify=False"
+            )
             category = requestsexceptions.InsecureRequestWarning
             if category:
                 # InsecureRequestWarning references a Warning class or is None
@@ -102,26 +104,15 @@ class _OpenStackCloudMixin:
         cache_class = self.config.get_cache_class()
         cache_arguments = self.config.get_cache_arguments()
 
-        self._resource_caches = {}
+        self._cache_expirations = dict()
 
         if cache_class != 'dogpile.cache.null':
             self.cache_enabled = True
-            self._cache = self._make_cache(
-                cache_class, cache_expiration_time, cache_arguments)
-            expirations = self.config.get_cache_expirations()
-            for expire_key in expirations.keys():
-                # Only build caches for things we have list operations for
-                if getattr(
-                        self, 'list_{0}'.format(expire_key), None):
-                    self._resource_caches[expire_key] = self._make_cache(
-                        cache_class, expirations[expire_key], cache_arguments)
-
-            self._SERVER_AGE = DEFAULT_SERVER_AGE
-            self._PORT_AGE = DEFAULT_PORT_AGE
-            self._FLOAT_AGE = DEFAULT_FLOAT_AGE
         else:
             self.cache_enabled = False
 
+            # TODO(gtema): delete it with the standalone cloud layer caching
+
             def _fake_invalidate(unused):
                 pass
 
@@ -142,21 +133,27 @@ class _OpenStackCloudMixin:
                 meth_obj = getattr(self, method, None)
                 if not meth_obj:
                     continue
-                if (hasattr(meth_obj, 'invalidate')
-                        and hasattr(meth_obj, 'func')):
+                if hasattr(meth_obj, 'invalidate') and hasattr(
+                    meth_obj, 'func'
+                ):
                     new_func = functools.partial(meth_obj.func, self)
                     new_func.invalidate = _fake_invalidate
                     setattr(self, method, new_func)
 
-        # If server expiration time is set explicitly, use that. Otherwise
-        # fall back to whatever it was before
-        self._SERVER_AGE = self.config.get_cache_resource_expiration(
-            'server', self._SERVER_AGE)
-        self._PORT_AGE = self.config.get_cache_resource_expiration(
-            'port', self._PORT_AGE)
-        self._FLOAT_AGE = self.config.get_cache_resource_expiration(
-            'floating_ip', self._FLOAT_AGE)
+        # Uncoditionally create cache even with a "null" backend
+        self._cache = self._make_cache(
+            cache_class, cache_expiration_time, cache_arguments
+        )
+        expirations = self.config.get_cache_expirations()
+        for expire_key in expirations.keys():
+            self._cache_expirations[expire_key] = expirations[expire_key]
 
+        # TODO(gtema): delete in next change
+        self._SERVER_AGE = 0
+        self._PORT_AGE = 0
+        self._FLOAT_AGE = 0
+
+        self._api_cache_keys = set()
         self._container_cache = dict()
         self._file_hash_cache = dict()
 
@@ -165,7 +162,8 @@ class _OpenStackCloudMixin:
         self._raw_clients = {}
 
         self._local_ipv6 = (
-            _utils.localhost_supports_ipv6() if not self.force_ipv4 else False)
+            _utils.localhost_supports_ipv6() if not self.force_ipv4 else False
+        )
 
     def connect_as(self, **kwargs):
         """Make a new OpenStackCloud object with new auth context.
@@ -197,7 +195,8 @@ class _OpenStackCloudMixin:
             config = openstack.config.OpenStackConfig(
                 app_name=self.config._app_name,
                 app_version=self.config._app_version,
-                load_yaml_config=False)
+                load_yaml_config=False,
+            )
         params = copy.deepcopy(self.config.config)
         # Remove profile from current cloud so that overridding works
         params.pop('profile', None)
@@ -304,7 +303,8 @@ class _OpenStackCloudMixin:
             app_name=self.config._app_name,
             app_version=self.config._app_version,
             discovery_cache=self.session._discovery_cache,
-            **params)
+            **params
+        )
 
         # Override the cloud name so that logging/location work right
         cloud_region._name = self.name
@@ -319,9 +319,8 @@ class _OpenStackCloudMixin:
         return dogpile.cache.make_region(
             function_key_generator=self._make_cache_key
         ).configure(
-            cache_class,
-            expiration_time=expiration_time,
-            arguments=arguments)
+            cache_class, expiration_time=expiration_time, arguments=arguments
+        )
 
     def _make_cache_key(self, namespace, fn):
         fname = fn.__name__
@@ -335,10 +334,11 @@ class _OpenStackCloudMixin:
             arg_key = ''
             kw_keys = sorted(kwargs.keys())
             kwargs_key = ','.join(
-                ['%s:%s' % (k, kwargs[k]) for k in kw_keys if k != 'cache'])
-            ans = "_".join(
-                [str(name_key), fname, arg_key, kwargs_key])
+                ['%s:%s' % (k, kwargs[k]) for k in kw_keys if k != 'cache']
+            )
+            ans = "_".join([str(name_key), fname, arg_key, kwargs_key])
             return ans
+
         return generate_key
 
     def _get_cache(self, resource_name):
@@ -355,7 +355,8 @@ class _OpenStackCloudMixin:
         return version
 
     def _get_versioned_client(
-            self, service_type, min_version=None, max_version=None):
+        self, service_type, min_version=None, max_version=None
+    ):
         config_version = self.config.get_api_version(service_type)
         config_major = self._get_major_version_id(config_version)
         max_major = self._get_major_version_id(max_version)
@@ -378,7 +379,9 @@ class _OpenStackCloudMixin:
                     " but shade understands a minimum of {min_version}".format(
                         config_version=config_version,
                         service_type=service_type,
-                        min_version=min_version))
+                        min_version=min_version,
+                    )
+                )
             elif max_major and config_major > max_major:
                 raise exc.OpenStackCloudException(
                     "Version {config_version} requested for {service_type}"
@@ -386,10 +389,13 @@ class _OpenStackCloudMixin:
                     " {max_version}".format(
                         config_version=config_version,
                         service_type=service_type,
-                        max_version=max_version))
+                        max_version=max_version,
+                    )
+                )
             request_min_version = config_version
             request_max_version = '{version}.latest'.format(
-                version=config_major)
+                version=config_major
+            )
             adapter = proxy._ShadeAdapter(
                 session=self.session,
                 service_type=self.config.get_service_type(service_type),
@@ -403,7 +409,8 @@ class _OpenStackCloudMixin:
                 prometheus_histogram=self.config.get_prometheus_histogram(),
                 influxdb_client=self.config.get_influxdb_client(),
                 min_version=request_min_version,
-                max_version=request_max_version)
+                max_version=request_max_version,
+            )
             if adapter.get_endpoint():
                 return adapter
 
@@ -415,12 +422,14 @@ class _OpenStackCloudMixin:
             endpoint_override=self.config.get_endpoint(service_type),
             region_name=self.config.get_region_name(service_type),
             min_version=min_version,
-            max_version=max_version)
+            max_version=max_version,
+        )
 
         # data.api_version can be None if no version was detected, such
         # as with neutron
         api_version = adapter.get_api_major_version(
-            endpoint_override=self.config.get_endpoint(service_type))
+            endpoint_override=self.config.get_endpoint(service_type)
+        )
         api_major = self._get_major_version_id(api_version)
 
         # If we detect a different version that was configured, warn the user.
@@ -436,7 +445,9 @@ class _OpenStackCloudMixin:
                 ' your config.'.format(
                     service_type=service_type,
                     config_version=config_version,
-                    api_version='.'.join([str(f) for f in api_version])))
+                    api_version='.'.join([str(f) for f in api_version]),
+                )
+            )
             self.log.debug(warning_msg)
             warnings.warn(warning_msg)
         return adapter
@@ -444,19 +455,22 @@ class _OpenStackCloudMixin:
     # TODO(shade) This should be replaced with using openstack Connection
     #             object.
     def _get_raw_client(
-            self, service_type, api_version=None, endpoint_override=None):
+        self, service_type, api_version=None, endpoint_override=None
+    ):
         return proxy._ShadeAdapter(
             session=self.session,
             service_type=self.config.get_service_type(service_type),
             service_name=self.config.get_service_name(service_type),
             interface=self.config.get_interface(service_type),
-            endpoint_override=self.config.get_endpoint(
-                service_type) or endpoint_override,
-            region_name=self.config.get_region_name(service_type))
+            endpoint_override=self.config.get_endpoint(service_type)
+            or endpoint_override,
+            region_name=self.config.get_region_name(service_type),
+        )
 
     def _is_client_version(self, client, version):
         client_name = '_{client}_client'.format(
-            client=client.replace('-', '_'))
+            client=client.replace('-', '_')
+        )
         client = getattr(self, client_name)
         return client._version_matches(version)
 
@@ -464,7 +478,8 @@ class _OpenStackCloudMixin:
     def _application_catalog_client(self):
         if 'application-catalog' not in self._raw_clients:
             self._raw_clients['application-catalog'] = self._get_raw_client(
-                'application-catalog')
+                'application-catalog'
+            )
         return self._raw_clients['application-catalog']
 
     @property
@@ -484,6 +499,7 @@ class _OpenStackCloudMixin:
         """Wrapper around pprint that groks munch objects"""
         # import late since this is a utility function
         import pprint
+
         new_resource = _utils._dictify_resource(resource)
         pprint.pprint(new_resource)
 
@@ -491,6 +507,7 @@ class _OpenStackCloudMixin:
         """Wrapper around pformat that groks munch objects"""
         # import late since this is a utility function
         import pprint
+
         new_resource = _utils._dictify_resource(resource)
         return pprint.pformat(new_resource)
 
@@ -527,7 +544,8 @@ class _OpenStackCloudMixin:
         return self.config.get_endpoint_from_catalog(
             service_type=service_type,
             interface=interface,
-            region_name=region_name)
+            region_name=region_name,
+        )
 
     @property
     def auth_token(self):
@@ -556,11 +574,11 @@ class _OpenStackCloudMixin:
 
     @property
     def current_project(self):
-        """Return a ``munch.Munch`` describing the current project"""
+        """Return a ``utils.Munch`` describing the current project"""
         return self._get_project_info()
 
     def _get_project_info(self, project_id=None):
-        project_info = munch.Munch(
+        project_info = utils.Munch(
             id=project_id,
             name=None,
             domain_id=None,
@@ -586,11 +604,11 @@ class _OpenStackCloudMixin:
 
     @property
     def current_location(self):
-        """Return a ``munch.Munch`` explaining the current cloud location."""
+        """Return a ``utils.Munch`` explaining the current cloud location."""
         return self._get_current_location()
 
     def _get_current_location(self, project_id=None, zone=None):
-        return munch.Munch(
+        return utils.Munch(
             cloud=self.name,
             # TODO(efried): This is wrong, but it only seems to be used in a
             # repr; can we get rid of it?
@@ -601,15 +619,14 @@ class _OpenStackCloudMixin:
 
     def _get_identity_location(self):
         '''Identity resources do not exist inside of projects.'''
-        return munch.Munch(
+        return utils.Munch(
             cloud=self.name,
             region_name=None,
             zone=None,
-            project=munch.Munch(
-                id=None,
-                name=None,
-                domain_id=None,
-                domain_name=None))
+            project=utils.Munch(
+                id=None, name=None, domain_id=None, domain_name=None
+            ),
+        )
 
     def _get_project_id_param_dict(self, name_or_id):
         if name_or_id:
@@ -634,7 +651,8 @@ class _OpenStackCloudMixin:
             if not domain_id:
                 raise exc.OpenStackCloudException(
                     "User or project creation requires an explicit"
-                    " domain_id argument.")
+                    " domain_id argument."
+                )
             else:
                 return {'domain_id': domain_id}
         else:
@@ -720,7 +738,8 @@ class _OpenStackCloudMixin:
             return self.config.get_session_endpoint(service_key, **kwargs)
         except keystoneauth1.exceptions.catalog.EndpointNotFound as e:
             self.log.debug(
-                "Endpoint not found in %s cloud: %s", self.name, str(e))
+                "Endpoint not found in %s cloud: %s", self.name, str(e)
+            )
             endpoint = None
         except exc.OpenStackCloudException:
             raise
@@ -731,17 +750,22 @@ class _OpenStackCloudMixin:
                     service=service_key,
                     cloud=self.name,
                     region=self.config.get_region_name(service_key),
-                    error=str(e)))
+                    error=str(e),
+                )
+            )
         return endpoint
 
     def has_service(self, service_key, version=None):
         if not self.config.has_service(service_key):
             # TODO(mordred) add a stamp here so that we only report this once
-            if not (service_key in self._disable_warnings
-                    and self._disable_warnings[service_key]):
+            if not (
+                service_key in self._disable_warnings
+                and self._disable_warnings[service_key]
+            ):
                 self.log.debug(
-                    "Disabling %(service_key)s entry in catalog"
-                    " per config", {'service_key': service_key})
+                    "Disabling %(service_key)s entry in catalog" " per config",
+                    {'service_key': service_key},
+                )
                 self._disable_warnings[service_key] = True
             return False
         try:
@@ -758,13 +782,79 @@ class _OpenStackCloudMixin:
         else:
             return False
 
+    def search_resources(
+        self,
+        resource_type,
+        name_or_id,
+        get_args=None,
+        get_kwargs=None,
+        list_args=None,
+        list_kwargs=None,
+        **filters
+    ):
+        """Search resources
+
+        Search resources matching certain conditions
+
+        :param str resource_type: String representation of the expected
+            resource as `service.resource` (i.e. "network.security_group").
+        :param str name_or_id: Name or ID of the resource
+        :param list get_args: Optional args to be passed to the _get call.
+        :param dict get_kwargs: Optional kwargs to be passed to the _get call.
+        :param list list_args: Optional args to be passed to the _list call.
+        :param dict list_kwargs: Optional kwargs to be passed to the _list call
+        :param dict filters: Additional filters to be used for querying
+            resources.
+        """
+        get_args = get_args or ()
+        get_kwargs = get_kwargs or {}
+        list_args = list_args or ()
+        list_kwargs = list_kwargs or {}
+
+        # User used string notation. Try to find proper
+        # resource
+        (service_name, resource_name) = resource_type.split('.')
+        if not hasattr(self, service_name):
+            raise exceptions.SDKException(
+                "service %s is not existing/enabled" % service_name
+            )
+        service_proxy = getattr(self, service_name)
+        try:
+            resource_type = service_proxy._resource_registry[resource_name]
+        except KeyError:
+            raise exceptions.SDKException(
+                "Resource %s is not known in service %s"
+                % (resource_name, service_name)
+            )
+
+        if name_or_id:
+            # name_or_id is definitely not None
+            try:
+                resource_by_id = service_proxy._get(
+                    resource_type, name_or_id, *get_args, **get_kwargs
+                )
+                return [resource_by_id]
+            except exceptions.ResourceNotFound:
+                pass
+
+        if not filters:
+            filters = {}
+
+        if name_or_id:
+            filters["name"] = name_or_id
+        list_kwargs.update(filters)
+
+        return list(
+            service_proxy._list(resource_type, *list_args, **list_kwargs)
+        )
+
     def project_cleanup(
         self,
         dry_run=True,
         wait_timeout=120,
         status_queue=None,
         filters=None,
-        resource_evaluation_fn=None
+        resource_evaluation_fn=None,
     ):
         """Cleanup the project resources.
 
@@ -788,16 +878,24 @@ class _OpenStackCloudMixin:
         if not status_queue:
             status_queue = queue.Queue()
         for service in self.config.get_enabled_services():
-            if hasattr(self, service):
-                proxy = getattr(self, service)
-                if (
-                    proxy
-                    and hasattr(proxy, get_dep_fn_name)
-                    and hasattr(proxy, cleanup_fn_name)
-                ):
-                    deps = getattr(proxy, get_dep_fn_name)()
-                    if deps:
-                        dependencies.update(deps)
+            try:
+                if hasattr(self, service):
+                    proxy = getattr(self, service)
+                    if (
+                        proxy
+                        and hasattr(proxy, get_dep_fn_name)
+                        and hasattr(proxy, cleanup_fn_name)
+                    ):
+                        deps = getattr(proxy, get_dep_fn_name)()
+                        if deps:
+                            dependencies.update(deps)
+            except (
+                exceptions.NotSupported,
+                exceptions.ServiceDisabledException,
+            ):
+                # Cloud may include endpoint in catalog but not
+                # implement the service or disable it
+                pass
         dep_graph = utils.TinyDAG()
         for k, v in dependencies.items():
             dep_graph.add_node(k)
@@ -811,18 +909,22 @@ class _OpenStackCloudMixin:
 
         for service in dep_graph.walk(timeout=wait_timeout):
             fn = None
-            if hasattr(self, service):
-                proxy = getattr(self, service)
-                cleanup_fn = getattr(proxy, cleanup_fn_name, None)
-                if cleanup_fn:
-                    fn = functools.partial(
-                        cleanup_fn,
-                        dry_run=dry_run,
-                        client_status_queue=status_queue,
-                        identified_resources=cleanup_resources,
-                        filters=filters,
-                        resource_evaluation_fn=resource_evaluation_fn
-                    )
+            try:
+                if hasattr(self, service):
+                    proxy = getattr(self, service)
+                    cleanup_fn = getattr(proxy, cleanup_fn_name, None)
+                    if cleanup_fn:
+                        fn = functools.partial(
+                            cleanup_fn,
+                            dry_run=dry_run,
+                            client_status_queue=status_queue,
+                            identified_resources=cleanup_resources,
+                            filters=filters,
+                            resource_evaluation_fn=resource_evaluation_fn,
+                        )
+            except exceptions.ServiceDisabledException:
+                # same reason as above
+                pass
             if fn:
                 self._pool_executor.submit(
                     cleanup_task, dep_graph, service, fn
@@ -831,9 +933,10 @@ class _OpenStackCloudMixin:
                 dep_graph.node_done(service)
 
         for count in utils.iterate_timeout(
-                timeout=wait_timeout,
-                message="Timeout waiting for cleanup to finish",
-                wait=1):
+            timeout=wait_timeout,
+            message="Timeout waiting for cleanup to finish",
+            wait=1,
+        ):
             if dep_graph.is_complete():
                 return
 
