@@ -131,7 +131,7 @@ construct a Connection with the ``CONF`` object and an authenticated Session.
 
     conn = connection.Connection(
         session=session,
-        oslo_config=CONF)
+        oslo_conf=CONF)
 
 From existing CloudRegion
 -------------------------
@@ -176,23 +176,27 @@ try to find it and if that fails, you would create it::
 Additional information about the services can be found in the
 :ref:`service-proxies` documentation.
 """
+import concurrent.futures
 import warnings
 import weakref
 
-import concurrent.futures
+try:
+    # For python 3.8 and later
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    # For everyone else
+    import importlib_metadata
 import keystoneauth1.exceptions
 import requestsexceptions
-import six
 
 from openstack import _log
 from openstack import _services_mixin
-from openstack.cloud import openstackcloud as _cloud
 from openstack.cloud import _accelerator
 from openstack.cloud import _baremetal
 from openstack.cloud import _block_storage
-from openstack.cloud import _compute
 from openstack.cloud import _clustering
 from openstack.cloud import _coe
+from openstack.cloud import _compute
 from openstack.cloud import _dns
 from openstack.cloud import _floating_ip
 from openstack.cloud import _identity
@@ -202,6 +206,8 @@ from openstack.cloud import _network_common
 from openstack.cloud import _object_store
 from openstack.cloud import _orchestration
 from openstack.cloud import _security_group
+from openstack.cloud import _shared_file_system
+from openstack.cloud import openstackcloud as _cloud
 from openstack import config as _config
 from openstack.config import cloud_region
 from openstack import exceptions
@@ -214,7 +220,8 @@ __all__ = [
 
 if requestsexceptions.SubjectAltNameWarning:
     warnings.filterwarnings(
-        'ignore', category=requestsexceptions.SubjectAltNameWarning)
+        'ignore', category=requestsexceptions.SubjectAltNameWarning
+    )
 
 _logger = _log.setup_logging('openstack')
 
@@ -243,7 +250,8 @@ def from_config(cloud=None, config=None, options=None, **kwargs):
     config = kwargs.pop('cloud_config', config)
     if config is None:
         config = _config.OpenStackConfig().get_one(
-            cloud=cloud, argparse=options, **kwargs)
+            cloud=cloud, argparse=options, **kwargs
+        )
 
     return Connection(config=config)
 
@@ -265,22 +273,28 @@ class Connection(
     _network_common.NetworkCommonCloudMixin,
     _object_store.ObjectStoreCloudMixin,
     _orchestration.OrchestrationCloudMixin,
-    _security_group.SecurityGroupCloudMixin
+    _security_group.SecurityGroupCloudMixin,
+    _shared_file_system.SharedFileSystemCloudMixin,
 ):
-
-    def __init__(self, cloud=None, config=None, session=None,
-                 app_name=None, app_version=None,
-                 extra_services=None,
-                 strict=False,
-                 use_direct_get=False,
-                 task_manager=None,
-                 rate_limit=None,
-                 oslo_conf=None,
-                 service_types=None,
-                 global_request_id=None,
-                 strict_proxies=False,
-                 pool_executor=None,
-                 **kwargs):
+    def __init__(
+        self,
+        cloud=None,
+        config=None,
+        session=None,
+        app_name=None,
+        app_version=None,
+        extra_services=None,
+        strict=False,
+        use_direct_get=False,
+        task_manager=None,
+        rate_limit=None,
+        oslo_conf=None,
+        service_types=None,
+        global_request_id=None,
+        strict_proxies=False,
+        pool_executor=None,
+        **kwargs
+    ):
         """Create a connection to a cloud.
 
         A connection needs information about how to connect, how to
@@ -366,24 +380,32 @@ class Connection(
         if not self.config:
             if oslo_conf:
                 self.config = cloud_region.from_conf(
-                    oslo_conf, session=session, app_name=app_name,
-                    app_version=app_version, service_types=service_types)
+                    oslo_conf,
+                    session=session,
+                    app_name=app_name,
+                    app_version=app_version,
+                    service_types=service_types,
+                )
             elif session:
                 self.config = cloud_region.from_session(
                     session=session,
-                    app_name=app_name, app_version=app_version,
+                    app_name=app_name,
+                    app_version=app_version,
                     load_yaml_config=False,
                     load_envvars=False,
                     rate_limit=rate_limit,
-                    **kwargs)
+                    **kwargs
+                )
             else:
                 self.config = _config.get_cloud_region(
                     cloud=cloud,
-                    app_name=app_name, app_version=app_version,
+                    app_name=app_name,
+                    app_version=app_version,
                     load_yaml_config=cloud is not None,
                     load_envvars=cloud is not None,
                     rate_limit=rate_limit,
-                    **kwargs)
+                    **kwargs
+                )
 
         self._session = None
         self._proxies = {}
@@ -409,6 +431,7 @@ class Connection(
         _object_store.ObjectStoreCloudMixin.__init__(self)
         _orchestration.OrchestrationCloudMixin.__init__(self)
         _security_group.SecurityGroupCloudMixin.__init__(self)
+        _shared_file_system.SharedFileSystemCloudMixin.__init__(self)
 
         # Allow vendors to provide hooks. They will normally only receive a
         # connection object and a responsible to register additional services
@@ -424,17 +447,37 @@ class Connection(
                     (package_name, function) = vendor_hook.rsplit(':')
 
                     if package_name and function:
-                        import pkg_resources
-                        ep = pkg_resources.EntryPoint(
-                            'vendor_hook', package_name, attrs=(function,))
-                        hook = ep.resolve()
+                        ep = importlib_metadata.EntryPoint(
+                            name='vendor_hook',
+                            value=vendor_hook,
+                            group='vendor_hook',
+                        )
+                        hook = ep.load()
                         hook(self)
                 except ValueError:
-                    self.log.warning('Hook should be in the entrypoint '
-                                     'module:attribute format')
-            except (ImportError, TypeError) as e:
-                self.log.warning('Configured hook %s cannot be executed: %s',
-                                 vendor_hook, e)
+                    self.log.warning(
+                        'Hook should be in the entrypoint '
+                        'module:attribute format'
+                    )
+            except (ImportError, TypeError, AttributeError) as e:
+                self.log.warning(
+                    'Configured hook %s cannot be executed: %s', vendor_hook, e
+                )
+
+        # Add additional metrics into the configuration according to the
+        # selected connection. We don't want to deal with overall config in the
+        # proxy, just pass required part.
+        if (
+            self.config._influxdb_config
+            and 'additional_metric_tags' in self.config.config
+        ):
+            self.config._influxdb_config[
+                'additional_metric_tags'
+            ] = self.config.config['additional_metric_tags']
+
+    def __del__(self):
+        # try to force release of resources and save authorization
+        self.close()
 
     @property
     def session(self):
@@ -465,7 +508,7 @@ class Connection(
         """
         # If we don't have a proxy, just instantiate Proxy so that
         # we get an adapter.
-        if isinstance(service, six.string_types):
+        if isinstance(service, str):
             service = service_description.ServiceDescription(service)
 
         # Directly invoke descriptor of the ServiceDescription
@@ -478,7 +521,7 @@ class Connection(
             setattr(
                 self.__class__,
                 attr_name.replace('-', '_'),
-                property(fget=getter)
+                property(fget=getter),
             )
         self.config.enable_service(service.service_type)
 
@@ -499,19 +542,21 @@ class Connection(
         try:
             return self.session.get_token()
         except keystoneauth1.exceptions.ClientException as e:
-            raise exceptions.raise_from_response(e.response)
+            raise exceptions.SDKException(e)
 
     @property
     def _pool_executor(self):
         if not self.__pool_executor:
             self.__pool_executor = concurrent.futures.ThreadPoolExecutor(
-                max_workers=5)
+                max_workers=5
+            )
         return self.__pool_executor
 
     def close(self):
         """Release any resources held open."""
         if self.__pool_executor:
             self.__pool_executor.shutdown()
+        self.config.set_auth_cache()
 
     def set_global_request_id(self, global_request_id):
         self._global_request_id = global_request_id
