@@ -20,11 +20,11 @@ import json
 import re
 
 from requests import exceptions as _rex
-import six
 
 
 class SDKException(Exception):
     """The base exception class for all exceptions this library raises."""
+
     def __init__(self, message=None, extra_data=None):
         self.message = self.__class__.__name__ if message is None else message
         self.extra_data = extra_data
@@ -36,6 +36,7 @@ OpenStackCloudException = SDKException
 
 class EndpointNotFound(SDKException):
     """A mismatch occurred between what the client and server expect."""
+
     def __init__(self, message=None):
         super(EndpointNotFound, self).__init__(message)
 
@@ -56,20 +57,25 @@ class InvalidRequest(SDKException):
 
 
 class HttpException(SDKException, _rex.HTTPError):
-
-    def __init__(self, message='Error', response=None,
-                 http_status=None,
-                 details=None, request_id=None):
+    def __init__(
+        self,
+        message='Error',
+        response=None,
+        http_status=None,
+        details=None,
+        request_id=None,
+    ):
         # TODO(shade) Remove http_status parameter and the ability for response
         # to be None once we're not mocking Session everywhere.
         if not message:
             if response is not None:
                 message = "{name}: {code}".format(
-                    name=self.__class__.__name__,
-                    code=response.status_code)
+                    name=self.__class__.__name__, code=response.status_code
+                )
             else:
                 message = "{name}: Unknown error".format(
-                    name=self.__class__.__name__)
+                    name=self.__class__.__name__
+                )
 
         # Call directly rather than via super to control parameters
         SDKException.__init__(self, message=message)
@@ -88,7 +94,7 @@ class HttpException(SDKException, _rex.HTTPError):
         if self.status_code is not None and (400 <= self.status_code < 500):
             self.source = "Client"
 
-    def __unicode__(self):
+    def __str__(self):
         # 'Error' is the default value for self.message. If self.message isn't
         # 'Error', then someone has set a more informative error message
         # and we should use it. If it is 'Error', then we should construct a
@@ -97,32 +103,46 @@ class HttpException(SDKException, _rex.HTTPError):
             return self.message
         if self.url:
             remote_error = "{source} Error for url: {url}".format(
-                source=self.source, url=self.url)
+                source=self.source, url=self.url
+            )
             if self.details:
                 remote_error += ', '
         if self.details:
-            remote_error += six.text_type(self.details)
+            remote_error += str(self.details)
 
         return "{message}: {remote_error}".format(
             message=super(HttpException, self).__str__(),
-            remote_error=remote_error)
-
-    def __str__(self):
-        return self.__unicode__()
+            remote_error=remote_error,
+        )
 
 
 class BadRequestException(HttpException):
     """HTTP 400 Bad Request."""
+
+    pass
+
+
+class ForbiddenException(HttpException):
+    """HTTP 403 Forbidden Request."""
+
     pass
 
 
 class ConflictException(HttpException):
     """HTTP 409 Conflict."""
+
+    pass
+
+
+class PreconditionFailedException(HttpException):
+    """HTTP 412 Precondition Failed."""
+
     pass
 
 
 class MethodNotSupported(SDKException):
     """The resource does not support this operation type."""
+
     def __init__(self, resource, method):
         # This needs to work with both classes and instances.
         try:
@@ -130,18 +150,23 @@ class MethodNotSupported(SDKException):
         except AttributeError:
             name = resource.__class__.__name__
 
-        message = ('The %s method is not supported for %s.%s' %
-                   (method, resource.__module__, name))
+        message = 'The %s method is not supported for %s.%s' % (
+            method,
+            resource.__module__,
+            name,
+        )
         super(MethodNotSupported, self).__init__(message=message)
 
 
 class DuplicateResource(SDKException):
     """More than one resource exists with that name."""
+
     pass
 
 
 class ResourceNotFound(HttpException):
     """No resource exists with that name or id."""
+
     pass
 
 
@@ -150,16 +175,19 @@ NotFoundException = ResourceNotFound
 
 class ResourceTimeout(SDKException):
     """Timeout waiting for resource."""
+
     pass
 
 
 class ResourceFailure(SDKException):
     """General resource failure."""
+
     pass
 
 
 class InvalidResourceQuery(SDKException):
     """Invalid query params for resource."""
+
     pass
 
 
@@ -171,7 +199,7 @@ def _extract_message(obj):
         # Ironic starting with Stein
         elif obj.get('faultstring'):
             return obj['faultstring']
-    elif isinstance(obj, six.string_types):
+    elif isinstance(obj, str):
         # Ironic before Stein has double JSON encoding, nobody remembers why.
         try:
             obj = json.loads(obj)
@@ -186,12 +214,16 @@ def raise_from_response(response, error_message=None):
     if response.status_code < 400:
         return
 
-    if response.status_code == 409:
-        cls = ConflictException
+    if response.status_code == 400:
+        cls = BadRequestException
+    elif response.status_code == 403:
+        cls = ForbiddenException
     elif response.status_code == 404:
         cls = NotFoundException
-    elif response.status_code == 400:
-        cls = BadRequestException
+    elif response.status_code == 409:
+        cls = ConflictException
+    elif response.status_code == 412:
+        cls = PreconditionFailedException
     else:
         cls = HttpException
 
@@ -205,6 +237,9 @@ def raise_from_response(response, error_message=None):
         try:
             content = response.json()
             messages = [_extract_message(obj) for obj in content.values()]
+            if not any(messages):
+                # Exception dict may be the root dict in projects that use WSME
+                messages = [_extract_message(content)]
             # Join all of the messages together nicely and filter out any
             # objects that don't have a "message" attr.
             details = '\n'.join(msg for msg in messages if msg)
@@ -212,8 +247,9 @@ def raise_from_response(response, error_message=None):
             details = response.text
     elif response.content and 'text/html' in content_type:
         # Split the lines, strip whitespace and inline HTML from the response.
-        details = [re.sub(r'<.+?>', '', i.strip())
-                   for i in response.text.splitlines()]
+        details = [
+            re.sub(r'<.+?>', '', i.strip()) for i in response.text.splitlines()
+        ]
         details = list(set([msg for msg in details if msg]))
         # Return joined string separated by colons.
         details = ': '.join(details)
@@ -225,8 +261,11 @@ def raise_from_response(response, error_message=None):
     request_id = response.headers.get('x-openstack-request-id')
 
     raise cls(
-        message=error_message, response=response, details=details,
-        http_status=http_status, request_id=request_id
+        message=error_message,
+        response=response,
+        details=details,
+        http_status=http_status,
+        request_id=request_id,
     )
 
 
@@ -236,6 +275,7 @@ class UnsupportedServiceVersion(Warning):
 
 class ArgumentDeprecationWarning(Warning):
     """A deprecated argument has been provided."""
+
     pass
 
 
@@ -257,3 +297,7 @@ class TaskManagerStopped(SDKException):
 
 class ServiceDisabledException(ConfigException):
     """This service is disabled for reasons."""
+
+
+class ServiceDiscoveryException(SDKException):
+    """The service cannot be discovered."""
