@@ -17,6 +17,7 @@ import os_service_types
 
 from openstack import _log
 from openstack import exceptions
+from openstack import proxy as proxy_mod
 
 __all__ = [
     'ServiceDescription',
@@ -26,7 +27,7 @@ _logger = _log.setup_logging('openstack')
 _service_type_manager = os_service_types.ServiceTypes()
 
 
-class _ServiceDisabledProxyShim(object):
+class _ServiceDisabledProxyShim:
     def __init__(self, service_type, reason):
         self.service_type = service_type
         self.reason = reason
@@ -35,11 +36,12 @@ class _ServiceDisabledProxyShim(object):
         raise exceptions.ServiceDisabledException(
             "Service '{service_type}' is disabled because its configuration "
             "could not be loaded. {reason}".format(
-                service_type=self.service_type, reason=self.reason or ''))
+                service_type=self.service_type, reason=self.reason or ''
+            )
+        )
 
 
-class ServiceDescription(object):
-
+class ServiceDescription:
     #: Dictionary of supported versions and proxy classes for that version
     supported_versions = None
     #: main service_type to use to find this service in the catalog
@@ -72,9 +74,8 @@ class ServiceDescription(object):
         """
         self.service_type = service_type or self.service_type
         self.supported_versions = (
-            supported_versions
-            or self.supported_versions
-            or {})
+            supported_versions or self.supported_versions or {}
+        )
 
         self.aliases = aliases or self.aliases
         self.all_types = [service_type] + self.aliases
@@ -83,10 +84,61 @@ class ServiceDescription(object):
         if instance is None:
             return self
         if self.service_type not in instance._proxies:
-            instance._proxies[self.service_type] = self._make_proxy(instance)
-            instance._proxies[self.service_type]._connection = instance
+            proxy = self._make_proxy(instance)
+            if not isinstance(proxy, _ServiceDisabledProxyShim):
+                # The keystone proxy has a method called get_endpoint
+                # that is about managing keystone endpoints. This is
+                # unfortunate.
+                try:
+                    endpoint = proxy_mod.Proxy.get_endpoint(proxy)
+                except IndexError:
+                    # It's best not to look to closely here. This is
+                    # to support old placement.
+                    # There was a time when it had no status entry
+                    # in its version discovery doc (OY) In this case,
+                    # no endpoints get through version discovery
+                    # filtering. In order to deal with that, catch
+                    # the IndexError thrown by keystoneauth and
+                    # set an endpoint_override for the user to the
+                    # url in the catalog and try again.
+                    self._set_override_from_catalog(instance.config)
+                    proxy = self._make_proxy(instance)
+                    endpoint = proxy_mod.Proxy.get_endpoint(proxy)
+                if instance._strict_proxies:
+                    self._validate_proxy(proxy, endpoint)
+                proxy._connection = instance
+            instance._proxies[self.service_type] = proxy
         return instance._proxies[self.service_type]
 
+    def _set_override_from_catalog(self, config):
+        override = config._get_endpoint_from_catalog(
+            self.service_type,
+            proxy_mod.Proxy,
+        )
+        config.set_service_value(
+            'endpoint_override',
+            self.service_type,
+            override,
+        )
+
+    def _validate_proxy(self, proxy, endpoint):
+        exc = None
+        service_url = getattr(proxy, 'skip_discovery', None)
+        try:
+            # Don't go too wild for e.g. swift
+            if service_url is None:
+                service_url = proxy.get_endpoint_data().service_url
+        except Exception as e:
+            exc = e
+        if exc or not endpoint or not service_url:
+            raise exceptions.ServiceDiscoveryException(
+                "Failed to create a working proxy for service {service_type}: "
+                "{message}".format(
+                    service_type=self.service_type,
+                    message=exc or "No valid endpoint was discoverable.",
+                )
+            )
+
     def _make_proxy(self, instance):
         """Create a Proxy for the service in question.
 
@@ -98,7 +150,8 @@ class ServiceDescription(object):
         if not config.has_service(self.service_type):
             return _ServiceDisabledProxyShim(
                 self.service_type,
-                config.get_disabled_reason(self.service_type))
+                config.get_disabled_reason(self.service_type),
+            )
 
         # We don't know anything about this service, so the user is
         # explicitly just using us for a passthrough REST adapter.
@@ -108,8 +161,6 @@ class ServiceDescription(object):
                 self.service_type,
                 allow_version_hack=True,
             )
-            # trigger EndpointNotFound exception if this is bogus
-            temp_client.get_endpoint()
             return temp_client
 
         # Check to see if we've got config that matches what we
@@ -138,13 +189,12 @@ class ServiceDescription(object):
                     " {service_type} is not known or supported by"
                     " openstacksdk. The resulting Proxy object will only"
                     " have direct passthrough REST capabilities.".format(
-                        version=version_string,
-                        service_type=self.service_type),
-                    category=exceptions.UnsupportedServiceVersion)
+                        version=version_string, service_type=self.service_type
+                    ),
+                    category=exceptions.UnsupportedServiceVersion,
+                )
         elif endpoint_override:
-            temp_adapter = config.get_session_client(
-                self.service_type
-            )
+            temp_adapter = config.get_session_client(self.service_type)
             api_version = temp_adapter.get_endpoint_data().api_version
             proxy_class = self.supported_versions.get(str(api_version[0]))
             if proxy_class:
@@ -159,12 +209,12 @@ class ServiceDescription(object):
                     " is not supported by openstacksdk. The resulting Proxy"
                     " object will only have direct passthrough REST"
                     " capabilities.".format(
-                        version=api_version,
-                        service_type=self.service_type),
-                    category=exceptions.UnsupportedServiceVersion)
+                        version=api_version, service_type=self.service_type
+                    ),
+                    category=exceptions.UnsupportedServiceVersion,
+                )
 
         if proxy_obj:
-
             if getattr(proxy_obj, 'skip_discovery', False):
                 # Some services, like swift, don't have discovery. While
                 # keystoneauth will behave correctly and handle such
@@ -173,6 +223,14 @@ class ServiceDescription(object):
                 return proxy_obj
 
             data = proxy_obj.get_endpoint_data()
+            if not data and instance._strict_proxies:
+                raise exceptions.ServiceDiscoveryException(
+                    "Failed to create a working proxy for service "
+                    "{service_type}: No endpoint data found.".format(
+                        service_type=self.service_type
+                    )
+                )
+
             # If we've gotten here with a proxy object it means we have
             # an endpoint_override in place. If the catalog_url and
             # service_url don't match, which can happen if there is a
@@ -181,7 +239,8 @@ class ServiceDescription(object):
             # so that subsequent discovery calls don't get made incorrectly.
             if data.catalog_url != data.service_url:
                 ep_key = '{service_type}_endpoint_override'.format(
-                    service_type=self.service_type.replace('-', '_'))
+                    service_type=self.service_type.replace('-', '_')
+                )
                 config.config[ep_key] = data.service_url
                 proxy_obj = config.get_session_client(
                     self.service_type,
@@ -194,16 +253,16 @@ class ServiceDescription(object):
         if version_string:
             version_kwargs['version'] = version_string
         else:
-            supported_versions = sorted([
-                int(f) for f in self.supported_versions])
+            supported_versions = sorted(
+                [int(f) for f in self.supported_versions]
+            )
             version_kwargs['min_version'] = str(supported_versions[0])
             version_kwargs['max_version'] = '{version}.latest'.format(
-                version=str(supported_versions[-1]))
+                version=str(supported_versions[-1])
+            )
 
         temp_adapter = config.get_session_client(
-            self.service_type,
-            allow_version_hack=True,
-            **version_kwargs
+            self.service_type, allow_version_hack=True, **version_kwargs
         )
         found_version = temp_adapter.get_api_major_version()
         if found_version is None:
@@ -214,35 +273,41 @@ class ServiceDescription(object):
                     " exists but does not have any supported versions.".format(
                         service_type=self.service_type,
                         cloud=instance.name,
-                        region_name=region_name))
+                        region_name=region_name,
+                    )
+                )
             else:
                 raise exceptions.NotSupported(
                     "The {service_type} service for {cloud}:{region_name}"
                     " exists but no version was discoverable.".format(
                         service_type=self.service_type,
                         cloud=instance.name,
-                        region_name=region_name))
-        proxy_class = self.supported_versions.get(str(found_version[0]))
-        if not proxy_class:
-            # Maybe openstacksdk is being used for the passthrough
-            # REST API proxy layer for an unknown service in the
-            # service catalog that also doesn't have any useful
-            # version discovery?
-            warnings.warn(
-                "Service {service_type} has no discoverable version."
-                " The resulting Proxy object will only have direct"
-                " passthrough REST capabilities.".format(
-                    service_type=self.service_type),
-                category=exceptions.UnsupportedServiceVersion)
-            return temp_adapter
+                        region_name=region_name,
+                    )
+                )
         proxy_class = self.supported_versions.get(str(found_version[0]))
         if proxy_class:
-            version_kwargs['constructor'] = proxy_class
-        return config.get_session_client(
-            self.service_type,
-            allow_version_hack=True,
-            **version_kwargs
+            return config.get_session_client(
+                self.service_type,
+                allow_version_hack=True,
+                constructor=proxy_class,
+                **version_kwargs
+            )
+
+        # No proxy_class
+        # Maybe openstacksdk is being used for the passthrough
+        # REST API proxy layer for an unknown service in the
+        # service catalog that also doesn't have any useful
+        # version discovery?
+        warnings.warn(
+            "Service {service_type} has no discoverable version."
+            " The resulting Proxy object will only have direct"
+            " passthrough REST capabilities.".format(
+                service_type=self.service_type
+            ),
+            category=exceptions.UnsupportedServiceVersion,
         )
+        return temp_adapter
 
     def __set__(self, instance, value):
         raise AttributeError('Service Descriptors cannot be set')
